# Transaction Types
AUTHORIZATION = "authorization"
CAPTURE_AUTHORIZATION = "capture-authorization"
CHECK_ENROLLMENT = "check-enrollment"
PURCHASE = "purchase"

# Transaction States
SUCCESS = "SUCCESS"
FAILED = "FAILED"

# Return Codes and Transaction Statuses
# https://document-center.wirecard.com/display/PTD/Return+Codes+and+Transaction+Statuses
CREATION_SUCCESS_ID = "201.0000"
CREATION_SUCCESS = "Creation Success"

CARD_ELIGIBLE_3D_ID = "200.1077"
CARD_ELIGIBLE_3D = "Card/Cardholder eligible for 3-D authentication process"
